% demo script for testing of algorithms from paper:
%
% Non-parametric Bayesian Models of Response Function in Dynamic Image Sequences
%
% submitted to Computer Vision and Image Understanding
%
% by Ondrej Tichy (otichy@utia.cas.cz) and Vaclav Smidl

clear all

%% data
load OTphantom_15_03_cviu.mat
% data matrix D and original sources
[p,n] = size(D);

kladPix = find(sum(D,2)>0); % find non-zeros pixels
kladObr = find(sum(D,1)>0); % find non-zeros images
D = D(kladPix,kladObr); % crop


%% starting point and parameters
r = 4; % expected number of sources
iterations = 300; % number of iterations
start = start_data(D,r); % structure with starting point of iterations

%% run selected algorithm
% select algortihm with response function prior:
% 1 - orthogonal
% 2 - sparse
% 3 - sparse differences
% 4 - Wishart
% 5 - Wishart with localization
algorithm = 5;

switch algorithm
    case 1
        [hat_A,hat_U,hat_b] = OrthogonalPrior_alg(D,r,start,iterations);
    case 2
        [hat_A,hat_U,hat_b] = SparsePrior_alg(D,r,start,iterations);
    case 3
        [hat_A,hat_U,hat_b] = SparseDiffPrior_alg(D,r,start,iterations);
    case 4
        [hat_A,hat_U,hat_b] = WishartPrior_alg(D,r,start,iterations);
    case 5
        [hat_A,hat_U,hat_b] = WishartLocalPrior_alg(D,r,start,iterations);
end

A = zeros(p,r);
U = zeros(n,r);
b = zeros(n,1);
A(kladPix,:) = hat_A;
U(kladObr',:) = hat_U;
b(kladObr') = hat_b;

%% report results
% print of source images A, response functions U, and input function b
% (direct time-activities X are available inside each algorithm)
figure;
colormap(invgray);
cols = r+1;
rows = 2;
for k = 1:r
    subplot(rows,cols,k)
        imagesc(reshape(A(:,k),sqrt(p),sqrt(p))');
        axis('square');  
    subplot(rows,cols,k+1+r)
        plot(U(:,k)./(max(U(:,k)))); % normed to [0;1] for simplicity
        ylim([0 1.1]);
        xlim([0 n]);
end
subplot(rows,cols,2*r+2)
    plot(b./max(b)); % normed to [0;1] for simplicity
    ylim([0 1.1]);
    xlim([0 n]);

display('OK');










